;
; CAPTFONT.ASM  -- C function capturing the current screen font
;                  Small memory model
;
;       int capturefont(void *buffer, int offset)
;
;       ... where offset is the address in EGA page 2 memory
;       returns the font in *buffer and the height as return value
;
;
;       void loadfont(void *buffer, int height, int bank)
;
;       loads a packed font to the given EGA font bank via BIOS
;
;

_TEXT           segment byte public 'CODE'
_TEXT           ends

DGROUP          group   _DATA,_BSS

_DATA           segment word public 'DATA'
d@              label   byte
d@w             label   word
Access_A000h    db 2,4,4,7,5,0,6,4,4,2
Protect_A000h   db 2,3,4,3,5,10h,6,0Ah,4,0
_DATA           ends

_BSS            segment word public 'BSS'
b@              label   byte
b@w             label   word
_BSS            ends

_TEXT           segment byte public 'CODE'

                assume  cs:_TEXT,ds:DGROUP,ss:DGROUP

_capturefont    proc near                       ; proc far in large-code models
                push bp
                mov bp,sp
                push si
                push di

                mov ah,0Fh                      ; Get CRT info
                int 10h
                cmp al,3                        ; Are we in text mode?
                jna Get_Font

                mov ax,3                        ; Otherwise set to text mode
                int 10h

Get_Font:       mov si,offset Access_A000h
                call Set_Registers

                mov dx,ds                       ; For large-data model we would
                mov es,dx                       ; save the segment of buffer
                                                ; Note that we save DS in DX

                mov ax,0A000h                   ; Set out source segment
                mov ds,ax                       ; (BP uses SS, to that's OK)

                mov di,[bp+4]                   ; Argument = target offset
                mov si,[bp+6]                   ; Source offset
                mov cx,4096                     ; 4096 words = 32*256 bytes

                rep movsw                       ; MOVSW quicker than MOVSB

                mov ds,dx                       ; Restore DS

                mov si,offset Protect_A000h
                call Set_Registers

                mov bh,2
                mov ax,1130h                    ; Get font information
                int 10h
                mov ax,cx                       ; Height of char matrix

                pop di
                pop si
                pop bp
                ret
_capturefont	endp


Set_Registers   proc near                       ; proc near in all models
                mov cx,2                        ; 2 sequencer register
                mov dx,3C4h                     ; Indexing register
                call Next_Register

                mov cx,3                        ; 3 graphics controller regs
                mov dl,0CEh                     ; Indexing register

Next_Register:  lodsb                           ; Get index
                out dx,al
                inc dx
                lodsb                           ; Get value
                out dx,al
                dec dx
                loop Next_Register
                ret
Set_Registers   endp


_loadfont       proc near                       ; proc far in large models
                push bp
                mov bp,sp

                mov ax,1110h                    ; Load user font
                mov bh,byte ptr [bp+6]          ; Screen height
                mov bl,byte ptr [bp+8]          ; Font bank number
                mov cx,256                      ; 256 characters defined
                xor dx,dx

                push ds                         ; If large-data ES = SEG buffer
                pop es

                mov bp,[bp+4]                   ; Offset of buffer

                int 10h

                pop bp
                ret
_loadfont       endp


                public  _capturefont, _loadfont

_TEXT           ends

_DATA           segment word public 'DATA'
s@              label   byte
_DATA           ends

                end
