#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>

#define mouseint() geninterrupt(0x33)		/* Converts to an INT 33H instr. */

#if defined(__COMPACT__) || defined(__LARGE__) || defined(__HUGE__)
#define LARGEDATA		1
#endif

int mousebuffersize(void)
/* Tells how many bytes that savemouse() will actually use */
{
	_AX = 0x15;

	mouseint();

	return( _BX + 1 );

}

void *savemouse(void *buffer, int *bufsize)
/* Saves the mouse state the buffer pointed to by (buffer) argument. A maximum
	of ( *bufsize ) bytes is used, ( *bufsize ) is then set to the number
	of bytes actually used (may vary with mouse driver). If buffer is too
	small, NULL is returned and *bufsize set to bytes needed, otherwise
	( buffer )is returned. If buffer is set to NULL, a malloc() will be
	performed to allocate memory.
	A NULL will be returned if malloc() failed, otherwise the block address.
	A flag will be stored in byte 0 to tell wheneither or not a free() should
	be performed by restoremouse(). If buffer == NULL, *bufsize will be set to
	size of buffer allocated.												*/
{

	int spacereq;
	char *buf1;

	_AX = 0x15;		/* How many bytes needed? */

	mouseint();

	spacereq = _BX;
	spacereq++;

	if ( buffer == NULL )
	{
		*bufsize = spacereq;
		buffer = malloc(spacereq);
		if ( buffer == NULL )
			return(NULL);
		*( (char *) buffer ) = 1;	/* malloc()ated buffer		*/
	}
	else
	{
		if ( spacereq > *bufsize )
			return(NULL);				/* ERROR: Not enough memory */

		*( (char *) buffer ) = 0;	/* User-allocated buffer	*/
	}

	*bufsize = spacereq;
	buf1 = (char *) buffer;
	buf1++;

#ifdef LARGEDATA

	_ES = FP_SEG(buf1);
	_DX = FP_OFF(buf1);

#else

	_ES = _DS;
	_DX = (unsigned) buf1;

#endif

_AX = 0x16;

mouseint();

return(buffer);

}

void restoremouse(void *buffer)
/* Restores the mouse settings as stored in buffer */
{
	char *buf1;

	buf1 = (char *) buffer;
	buf1++;


#ifdef LARGEDATA

	_ES = FP_SEG(buf1);
	_DX = FP_OFF(buf1);

#else

	_ES = _DS;
	_DX = (unsigned) buf1;

#endif

	_AX = 0x17;

	mouseint();

	if ( *( (char *) buffer ) )
		free(buffer);

}

