#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>

#define mouseint() geninterrupt(0x33)		/* Converts to an INT 33H instr. */

#if defined(__COMPACT__) || defined(__LARGE__) || defined(__HUGE__)
#define LARGEDATA		1
#endif

typedef void (far *mou_vect)();

void setmousetrap(int mask, mou_vect vector, int far *oldmask,
	mou_vect *oldvector )
/* This function declares a handler for use by the mouse routine. The argument
	mask is composed of

	MOU_MOVE	... cursor position changed (0x01)
	MOU_PRLEFT	... left button pressed		(0x02)
	MOU_PRRIGHT	... right button pressed	(0x08)
	MOU_RELEFT	... left button released	(0x04)
	MOU_RERIGHT	... right button released	(0x10)

	which can be combined by logical OR. To disable, the value

	MOU_NOTHING	... never do anything		(0x00)

	can be used. (All of these are part of the enum type MOU_MASK.)

	The argument vector is a pointer on the handler function, which *must* be
	declared as void interrupt (see Turbo C manual).


	When the function is entered, the following data are in the register
	pseudovariables:

	_AX	... the condition mask
	_BX	... current button state
	_CX ... cursor's horizontal coordinate
	_DX ... cursor's vertical coordinate
	_DI ... horizontal counts
	_SI ... vertical counts

	Remember that these variables are highly volatile. Store any needed infor-
	mation in regular variables immediately, or use the Turbo C interrupt-
	passing through arguments of interrupt routines (see manual).

	The arguments oldmask and oldvec are pointers to buffers where the
	previous mask/vector can be stored, in order to later be restored by
	another call to setmousetrap().

	All symbolic constants are defined in MOUSE.H .	*/

{

	_ES = FP_SEG(vector);
	_DX = FP_OFF(vector);

	_CX = mask;
	_AX = 0x14;

	mouseint();

	*oldmask = _CX;

	*oldvector = MK_FP(_ES,_DX);
}
