                    NAME VideoID

;
; Graphics subsystem detector from "Programmer's Guide to PC & PS/2
; Video Systems".
;
; Corresponds to the C prototype:
;
; void far cdecl VideoID(VIDstruct far *video[2]);
;
; typedef struct
; {
;      char videosystem;
;      char monitor;
; }
; VIDstruct;
;
; video[0] = Current active display subsystem
; video[1] = Passive video system (if any)
;
; Symbolic constants for different displays, prototype & typedef in file
; videoid.h .
;

ARGpVID_Offs    EQU word ptr [bp+6]
ARGpVID_Seg     EQU word ptr [bp+8]


VIDstruct       STRUC

Video0Type      DB ?
Display0Type    DB ?
Video1Type      DB ?
Display1Type    DB ?

VIDstruct       ENDS


Device0         EQU word ptr es:Video0Type[di]
Device1         EQU word ptr es:Video1Type[di]

MDA             EQU 1
CGA             EQU 2
EGA             EQU 3
MCGA            EQU 4
VGA             EQU 5
HGC             EQU 80h
HGCPlus         EQU 81h
InColor         EQU 82h

MDADisplay      EQU 1
CGADisplay      EQU 2
EGAColorDisplay EQU 3
VGAMonoDisplay  EQU 4
VGAColorDisplay EQU 5

TRUE            EQU 1
FALSE           EQU 0


_TEXT           segment byte public 'CODE'
                assume cs:_TEXT, ds:_TEXT       ; We do it this way
                                                ; in order to permit the usage
                                                ; of ALL memory models!
                PUBLIC _VideoID

_VideoID        proc far

                assume cs:_TEXT, ds:_TEXT

                push bp
                mov bp,sp
                push si
                push di
                push ds

                push cs                         ; Set DS equal to CS
                pop ds

                mov di,ARGpVID_Offs
                mov ax,ARGpVID_Seg
                mov es,ax                       ; ES:DI points to data block

                mov Device0, 0                  ; No devices found yet
                mov Device1, 0

                mov byte ptr CGAflag, TRUE
                mov byte ptr EGAflag, TRUE
                mov byte ptr Monoflag, TRUE

                mov cx, NumberOfTests
                mov si, offset TestSequence

L01:
                lodsb                           ; AL = possibility flag
                test al, al
                lodsw                           ; Load subroutine address
                jz L02                          ; Skip if not a possibility

                push si
                push cx
                call ax                         ; Call testing subroutine
                pop cx
                pop si
L02:
                loop L01                        ; Do the rest of the tests

                call FindActive                 ; Check which one is active
                pop ds                          ; Restore all registers
                pop di
                pop si
                pop bp
                ret                             ; and return

_VideoID        endp

;
; FindPS2       = Check if a VGA or MCGA is installed, check device ID
;
FindPS2         proc near

                mov ax,1A00h
                int 10h                         ; Call for video info

                cmp al,1Ah
                jne L13                         ; If no VGA/MCGA, return

                mov cx,bx
                xor bh,bh                       ; BX = code for active system
                or ch,ch
                jz L11                          ; Only one system?

                mov bl,ch                       ; BX = code for inactive system
                add bx,bx
                mov ax,[bx+offset DCCTable]     ; Code in our format

                mov Device1,ax

                mov bl,cl
                xor bh,bh                       ; BX = code for active again
L11:
                add bx,bx
                mov ax,[bx+offset DCCTable]

                mov Device0,ax

                mov byte ptr CGAflag, FALSE     ; These are now impossible
                mov byte ptr EGAflag, FALSE
                mov byte ptr Monoflag, FALSE

                lea bx,Video0Type[di]           ; If MDA reported...
                cmp byte ptr es:[bx], MDA
                je L12

                lea bx,Video1Type[di]
                cmp byte ptr es:[bx], MDA
                jne L13
L12:
                mov word ptr es:[bx], 0         ; ...it might be Hercules
                mov byte ptr Monoflag, TRUE
L13:
                ret

FindPS2         endp

;
; FindEGA       = Test for the presence of an EGA BIOS
;
FindEGA         proc near

                mov bl, 10h                     ; Return EGA info
                mov ah, 12h
                int 10h                         ; Call EGA BIOS if present

                cmp bl,10h                      ; BL changed?
                je L22                          ; If not, no EGA BIOS

                mov al,cl
                shr al,1
                mov bx,offset EGADisplays
                xlat
                mov ah,al
                mov al,EGA
                call FoundDevice

                cmp ah,MDADisplay
                je L21
                mov CGAflag, FALSE              ; No CGA with colour EGA
                jmp short L22
L21:
                mov Monoflag, FALSE             ; No MDA/Herc with mono EGA
L22:
                ret

FindEGA         endp

;
; FindCGA       = Detect the presence of a CGA adapter
;
FindCGA         proc near

                mov dx,3D4h                     ; CGA's CRTC port
                call Find6845
                jne L31                         ; No 6845 at 3D4h = No CGA

                mov al,CGA
                mov ah,CGADisplay
                call FoundDevice
L31:
                ret

FindCGA         endp

;
; FindMono      = Detect the presence of and identify MDA, HGC, HGC+ and InColor
;
FindMono        proc near

                mov dx,3B4h
                call Find6845
                jne L44

                mov dl,0BAh
                in al,dx
                and al,80h
                mov ah,al

                mov cx,8000h                    ; Do the test 32768 times!!
L41:
                in al,dx
                and al,80h
                cmp ah,al
                loope L41

                jne L42

                mov al,MDA                      ; If no Hercules, it's MDA
                mov ah,MDADisplay
                call FoundDevice
                jmp short L44
L42:
                in al,dx
                mov dl,al
                and dl,70h

                mov ah,MDADisplay               ; Assume the Hercules is mono

                mov al,HGCPlus                  ; Is it a HGC+?
                cmp dl,10h
                je L43

                mov al,HGC                      ; Is it a regular HGC?
                cmp dl,50h
                jne L43

                mov al,InColor                  ; No... it's an InColor
                mov ah,EGAColorDisplay          ; which has an EGA monitor
L43:
                call FoundDevice
L44:
                ret

FindMono        endp

;
; Find6845      = Test for the presence of an 6845 CRTC (MDA/CGA/Hercules)
;
Find6845        proc near

                mov al,0Fh
                out dx,al
                inc dx

                in al,dx
                mov ah,al
                mov al,66h
                out dx,al

                mov cx,100h                     ; Delay for 6845 to get it
L51:            loop L51

                in al,dx
                xchg ah,al

                out dx,al

                cmp ah,66h                      ; ZF set if found
                ret

Find6845        endp

;
; FindActive    = Determines which of two system that is online
;
FindActive      proc near

                cmp Device1, 0
                je L63

                cmp Device0, 4
                jge L63
                cmp Device1, 4
                jge L63

                mov ah,0Fh
                int 10h

                and al, 7
                cmp al, 7
                je L61

                cmp es:Display0Type[di],MDADisplay
                jne L63
                jmp short L62
L61:
                cmp es:Display0Type[di],MDADisplay
                je L63
L62:
                mov ax,Device0
                xchg ax,Device1
                mov Device0,ax
L63:
                ret

FindActive      endp

;
; FoundDevice   = Update the video system list
;
FoundDevice     proc near

                lea bx,Video0Type[di]
                cmp byte ptr es:[bx],0
                je L71

                lea bx,Video1Type[di]
L71:
                mov es:[bx],ax
                ret

FoundDevice     endp

;
; === And the data area, in _TEXT for multimodel compatibility only
;

EGADisplays     db      CGADisplay
                db      EGAColorDisplay
                db      MDADisplay
                db      CGADisplay
                db      EGAColorDisplay
                db      MDADisplay

DCCTable        db      0,0
                db      MDA,MDADisplay
                db      CGA,CGADisplay
                db      0,0
                db      EGA,EGAColorDisplay
                db      EGA,MDADisplay
                db      0,0
                db      VGA,VGAMonoDisplay
                db      VGA,VGAColorDisplay
                db      0,0
                db      MCGA,EGAColorDisplay
                db      MCGA,VGAMonoDisplay
                db      MCGA,VGAMonoDisplay

TestSequence    db      ?
                dw      FindPS2
EGAFlag         db      ?
                dw      FindEGA
CGAFlag         db      ?
                dw      FindCGA
Monoflag        db      ?
                dw      FindMono

NumberOfTests   equ ($-TestSequence)/3

_TEXT           ENDS

                END

