/* WINDOW02.C contains window-handling functions for Turbo C, part 2 */

#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <crtplus.h>

enum WSTAT { W_OK, W_NOTINIT, W_NOSPACE, W_EMPTY, W_RANGE, W_ILLEGAL };
enum WMODE { W_NORMAL, W_SPECIAL };
enum WSTYLE { W_SINGLE, W_DOUBLE, W_DVERT, W_DHORIZ, W_NOBORDER = 64,
    W_SHADOW = 128 };

extern int WindowColor;
extern enum WSTYLE WindowStyle;


struct WindowSysStruct
{
	unsigned *storage;
	unsigned *bufptr;
	int wsize;
	int wlim;
	int spaceleft;
    int maxcol;
    int maxline;
    int shadowwid;
    int colour;
};


extern struct WindowSysStruct _WindowSys;


#define order(a,b) if (a>b) { i = a; a=b; b=i; }
#define TESTINIT if ( _WindowSys.bufptr == NULL ) return(W_NOTINIT)
#define xwindows() ( _WindowSys.spaceleft < _WindowSys.wsize )

#define WS      _WindowSys


static char lineelem[4][6] =
	{
		{ '', '', '', '', '', '' },	/* W_SINGLE */
		{ '', '', '', '', '', '' },	/* W_DOUBLE */
		{ '', '', '', '', '', '' },	/* W_DVERT  */
		{ '', '', '', '', '', '' }	/* W_DHORIZ */
	};



enum WSTAT makewindow(int x, int y, int xsize, int ysize, enum WMODE mode)
/* Allocate a window with coordinates above. If (mode == W_SPECIAL) then allow
   access to limited space, otherwise not. Coordinates (x,y) INCLUDE border,
   (xsize,ysize) defines available space WITHOUT border.
   The window is allocated by calling zonewindow().
   Uses external variables WindowColor and WindowStyle for setting the colour
   and style of the window. If WindowBorder contains W_SHADOW option, the
   window area is expanded to include a shadow - ONLY POSSIBLE IF SPACE IS
   GIVEN, or W_ILLEGAL will be returned.

   If WindowBorder contains W_NOBORDER, the whole screen area allocated will
   be available, and NO PROCESSING WHATSOEVER will be done. (Exception:
   W_NOBORDER can be combined with W_SHADOW; then a shadow will as well be
   produced).

   Exit codes:
   W_OK			Window created
   W_RANGE      Window coordinates out of range, or (xsize,ysize) < (1,1)
   W_NOSPACE	Buffer full (if mode == W_NORMAL : limited space may exist)
   W_NOTINIT	initwindow() not successfully called
   */

{
	int i, j, memreq, hspan, vspan, shadow, border;
	struct
	{
		unsigned char chr;
		unsigned char atr;
	}
	pel;
	char *linseg;
	char ctemp;

	TESTINIT;

	shadow = ( ( WindowStyle & W_SHADOW ) != 0 );
	border = ( ( WindowStyle & W_NOBORDER ) == 0 );
	linseg = lineelem[WindowStyle & 3];

	hspan = xsize + (border ? 3 : -1) + (shadow ? WS.shadowwid : 0);
	vspan = ysize + (border ? 1 : -1) + shadow;

    if ( xsize<1 || ysize<1 ||  y + vspan > WS.maxline ||
        y < 1 ||  x + hspan > WS.maxcol || x < 1 )
        return(W_RANGE);

    memreq = (hspan+1) * (vspan+1) + 8;

    if ( (WS.spaceleft - memreq) < ( mode ? 0 : WS.wlim ) )
        return(W_NOSPACE);

    WS.spaceleft -= memreq;

    gettext(x, y, x+hspan, y+vspan, WS.bufptr);
    WS.bufptr += (memreq-8);

    *(WS.bufptr++) = WindowStyle;
    *(WS.bufptr++) = WindowColor;
    *(WS.bufptr++) = ysize;
    *(WS.bufptr++) = xsize;
    *(WS.bufptr++) = vspan;
    *(WS.bufptr++) = hspan;
    *(WS.bufptr++) = y;
    *(WS.bufptr++) = x;

	if ( shadow )
	{
		hspan -= WS.shadowwid;
		vspan--;
	}

	window(x, y, x + hspan, y + vspan);

	if ( border )
	{
		textattr(WindowColor);
		clrscr();

		gotoxy(1,1);
		putch( linseg[0] );

		ctemp = linseg[4];
		for ( i = xsize + 2 ; i ; i-- )
			putch(ctemp);

		putch( linseg[1] );

		ctemp = linseg[5];
		for ( i = ysize ; i>1 ; i-- )
		{
			gotoxy(1,i);
			putch(ctemp);
			gotoxy(xsize+4,i);
			putch(ctemp);
		}

		gotoxy(1,ysize+1);
		putch( linseg[2] );

		ctemp = linseg[4];
		for ( i = xsize+2 ; i ; i-- )
			putch(ctemp);

		putch( linseg[3] );
	}

	if ( shadow )
	{
		j = y + vspan + 1;

        for ( i = WS.shadowwid + 1 ; i <= hspan + 1 ; i++ )
		{
			gettext(i,j,i,j,&pel);
			pel.atr &= 0x87;					/* No backgr., no intense	*/
			puttext(i,j,i,j,&pel);
		}

		for ( i = x + hspan + WS.shadowwid ; i > x + hspan ; i-- )
		{
			for ( ; j >= y + WS.shadowwid ; j-- )
			{
				gettext(i,j,i,j,&pel);
				pel.atr &= 0x87;
				puttext(i,j,i,j,&pel);
			}
		}
	}

	if ( border )
	{
		window(x+2, y+1, x+xsize+2, y+ysize);
	}
	else
	{
		window(x,y,x+xsize-1,y+ysize-1);
	}

	return(W_OK);
}


enum WSTAT killwindow(void)
/* Erase the latest created window and restore window before as current
   Exit codes:
   W_OK		Operation successfully completed
   W_EMPTY	No window to delete
   */
{
	int x, y, hspan, vspan;

	TESTINIT;

	if ( !xwindows() )
		return(W_EMPTY);

    x = *(--WS.bufptr);
    y = *(--WS.bufptr);
    hspan = *(--WS.bufptr);
	vspan = *(--WS.bufptr);
	WS.bufptr -= ( (hspan+1) * (vspan+1) + 8 );
    puttext(x, y, x+hspan, y+vspan, WS.bufptr);

    WS.spaceleft += (hspan+1) * (vspan+1) + 8;

    if ( WS.wsize <= WS.spaceleft )
	{
        window (1,1,WS.maxcol,WS.maxline); /* Bakgrunden */
		return(W_OK);
	}



	window( WS.bufptr[-1]+3, WS.bufptr[-2]+1,     /* NEEDS CORRECTION */
			WS.bufptr[-3]-3, WS.bufptr[-4]-1  );

	return (W_OK);

}
